/*******************************************************************************************
    Method to be called on on iframe load 
********************************************************************************************/
function OnIframeLoad() 
{
	var iframe = document.getElementById('iframe');
	iframe.contentWindow.addEventListener('message', new DAM_ConnectFrame().messageEventHandler);
}

/**
 * @class DAM_ConnectFrame
 * This is the entry point to the DAM_ConnectFrame infrastructure.
 * Instantiate this object and use it to:
 *
 * @return A new \c DAM_ConnectFrame object
 */
function DAM_ConnectFrame() { }

/*******************************************************************************************
    Event Handler for postMessage 
********************************************************************************************/
DAM_ConnectFrame.prototype.messageEventHandler = function(event) 
{
	console.log("Inside Message Response Handler");	
	console.log(event);
	//if (event.origin == API_CONSTANTS.origin) {
		//console.log('origin matched');
		new DAM_ConnectFrame().postMessageCases(event);
	//}
}

/*******************************************************************************************
  	Case handling of eventtype of postMessage response handler  
********************************************************************************************/
DAM_ConnectFrame.prototype.postMessageCases = function(event)
{
	console.log(event.data);
	
	switch (event.data.eventType) 
	{
		case 'resizePanel':
			this.onResize(event.data);			
		break;
		
		case 'downloadDocument':
			this.onDownload(event.data, CONSTANTS.DOCUMENT);
		break;
		
		case 'downloadAsset':
			this.onDownload(event.data, CONSTANTS.IMAGE_FILE);
		break;
		
        case 'uploadAsset':
            console.log(event.data);
            alert('Upload Asset Event : ' + JSON.stringify(event.data));
		case 'closeDocument':
			
			this.closeDocument();
		break;
		
		case 'uploadDocument':
			CONSTANTS.MAIN_SCRIPT.uploadDocument(event.data);
		break;
		
		case 'loginSuccess':
			this.onLoginSucess(event.data);
		break;
		
		case 'logoutSuccess':
			this.onLogout(event.data);
		break;
	}
}

/*******************************************************************************************
  	Method called after Login button click from iframe
********************************************************************************************/
DAM_ConnectFrame.prototype.onLoginSucess = function(data)
{
	console.log("After Login Success");
	//CONSTANTS.CS.resizeContent(1000, 750);
}

/*******************************************************************************************
    Method called after Logout button click from iframe
********************************************************************************************/
DAM_ConnectFrame.prototype.onLogout = function(data)
{
	console.log("After Logout Success");	
	//CONSTANTS.CS.resizeContent(350, 650);
}

/*******************************************************************************************
Method called after Logout button click from iframe
********************************************************************************************/
DAM_ConnectFrame.prototype.onResize = function(data)
{
	
	if(data.items.length)
	{
		var dataItem = data.items[0];
		var width = parseInt(dataItem.width);
		var height = parseInt(dataItem.height);
		
		if(width > 300 && height > 300)
			new CSInterface().resizeContent(width, height);
		else
			alert("Allowed minimum dimension is 300 X 300");
		
	}
}

DAM_ConnectFrame.prototype.closeDocument = function()
{
	var script = '$._ext_common.closeDocument()';
	
	CONSTANTS.CS.evalScript(script, function (scritpResp)
	{
		scritpResp = JSON.parse(scritpResp);
		if(!scritpResp.success)
			alert(scritpResp.data);				
	});
}

/*******************************************************************************************
Method called after Download click from iframe
********************************************************************************************/
DAM_ConnectFrame.prototype.onDownload = function(data, type)
{
	console.log("After Download clicked");
	
	if(!(CONSTANTS.DOWNLOAD_PATH && CONSTANTS.DOWNLOAD_PATH.trim().length)) {
		console.log('document path is not set');
		alert('Please set Download Path in settings');
		return;
	}

	var downloadItem = data.items[0];
	var downloadUrl = downloadItem.url;
	
	var downloadPath = CONSTANTS.DOWNLOAD_PATH + CONSTANTS.PATH_SEPARATOR;
	
	switch(type)
	{
		case CONSTANTS.DOCUMENT : downloadPath += downloadItem.documentName;break;
		case CONSTANTS.IMAGE_FILE : downloadPath += downloadItem.assetName; break;
	}
	
	var cs = CONSTANTS.CS;
	
	console.log(downloadPath);
	console.log(downloadUrl);

	var fs = cep_node.require('fs');
	var file = fs.createWriteStream(downloadPath);

	var https = cep_node.require(downloadUrl.startsWith("https") ? 'https' : 'http');
	console.log('downloading ...');
	var request = https.get(downloadUrl, function (response) 
	{
		response.pipe(file);

		file.on('finish', function ()
		{
			console.log('file downloaded');
			response.unpipe(file);
			file.close();

			var script = undefined;
			
			switch(type)
			{
				case CONSTANTS.IMAGE_FILE :
					console.log('placing image file');
					setTimeout(function()
					{
						switch(CONSTANTS.CURRENT_APP)
						{
							case "IDSN" : script = '$._ext_IDSN.placeFile(' +  JSON.stringify(downloadPath) + ')';
										cs.evalScript(script, function (resp)
										{
											resp = JSON.parse(resp);
											console.log(resp);
											if(resp.success)
											{
												// post a message as the place gun is loaded
												var objMessage = {
														'eventType' : 'assetReadyToPlace',																												
														'path' : downloadPath
												};
												
												console.log(objMessage);
												document.getElementById('iframe').contentWindow
												.postMessage(objMessage, CONSTANTS.IFRAME_SRC);
											}
										});
										break;
							
							case "PHSP" :
							case "PHXS" : script = '$._ext_PHXS.placeFile(' +  JSON.stringify(downloadPath) + ')';
										cs.evalScript(script, function (resp)
										{
											console.log(resp);
											console.log('placeFile called on onDownload');
										});
										break;
						}
					}, 2000);
				
				break;

				case CONSTANTS.DOCUMENT :
					console.log('opening document');

					var assetInfo = {
							assetName : downloadItem.documentName,
							assetId : downloadItem.assetId,
							version : downloadItem.versionId,
							path : downloadPath
						};
				
					console.log(assetInfo);
					
					switch(CONSTANTS.CURRENT_APP)
					{
						case "IDSN" : script = '$._ext_IDSN.openDocument(' + JSON.stringify(assetInfo) + ')';
							break;
						
						case "PHSP" :
						case "PHXS" : script = '$._ext_PHXS.openDocument(' + JSON.stringify(assetInfo) + ')';
							break;
					}
					
					cs.evalScript(script, function (resp)
					{
						console.log(resp);
						console.log('openDocument called on onDownload');
					});

				break;
			}
		});// finished downloading
	}).on('error', function (err)
	{
		fs.unlink(downloadPath);
	});
}