/**
 * @class MainScript This is the entry point. Instantiate this object and use it to:
 * 
 * @return A new \c MainScript object
 */
function MainScript() { }

/*******************************************************************************
 * Method to be called on on body load
 ******************************************************************************/
function onLoaded()
{
	var cs = new CSInterface();

	CONSTANTS.CS = cs;
	CONSTANTS.MAIN_SCRIPT = new MainScript();
	CONSTANTS.PATH_SEPARATOR = (cs.getOSInformation().match(/Mac OS/gi) != null) ? "/" : "\\";
	CONSTANTS.MAIN_SCRIPT.initiliseJS(cs);
}

/*******************************************************************************
* Start of the plugin
******************************************************************************/
MainScript.prototype.initiliseJS = function(cs)
{
	this.loadJSX(cs);	
	this.registerEvents(cs); // common standard events for InDesign and Photoshop 
	this.createMenuXML(cs);
	this.readSettings();
};

/*******************************************************************************
 * Load JSX file into the scripting context of the product.
 ******************************************************************************/
MainScript.prototype.loadJSX = function(cs)
{
	console.log('to load jsx files');
	
	var appName = cs.hostEnvironment.appName;
	CONSTANTS.CURRENT_APP = appName;
	
	var extensionRoot = cs.getSystemPath(SystemPath.EXTENSION);
	var jsxFolder = extensionRoot + "/jsx"; 
	cs.evalScript('$._ext.evalFiles("' + jsxFolder + '")');

	// depending upon the application is InDesign or PhotoShop we need to load JSX files and register specific events
	if (appName == "IDSN")
	{
		// in case of InDesign, we need to load plug plug library first
		this.loadPlugPlugLibrary(extensionRoot);
		
		jsxFolder = jsxFolder + "/IDSN/";
		console.log("loading jsx for indesign");
		console.log(jsxFolder);
		
		cs.evalScript('$._ext.evalFiles("' + jsxFolder + '")');
		
		this.addIDSNEventListeners(cs);
	}
	else if (appName == "PHSP" || appName == "PHXS")
	{
		jsxFolder = jsxFolder + "/PHXS/";
		console.log("loading jsx for photoshop");
		console.log(jsxFolder);
		cs.evalScript('$._ext.evalFiles("' + jsxFolder + '")');
		
		var extensionId = cs.getExtensionID();
		this.addPHXSEventListeners(cs, extensionId);
		cs.addEventListener("com.adobe.PhotoshopJSONCallback" + extensionId, this.handlePhotoshopEvent);
	}
}

/*******************************************************************************
 * Method to register events for the plug-in
 ******************************************************************************/
MainScript.prototype.registerEvents = function(cs) { }

/*-----------------INDESIGN SPECIFIC CODE STARTS HERE ----------------------------*/

/*******************************************************************************
 * Method to load plug plug library for InDesign Event capturing.
 ******************************************************************************/
MainScript.prototype.loadPlugPlugLibrary = function()
{
	console.log('loading plugplug..');
	
	var cs =  CONSTANTS.CS;
	var osInformation = cs.getOSInformation();
	var extensionPath = cs.getSystemPath(SystemPath.EXTENSION); 
	var plugPlugFile;
	
	if (osInformation.match(/Mac OS/gi) != null)
		plugPlugFile = extensionPath + "/lib/PlugPlugExternalObject-Mac/osx10_64/PlugPlugExternalObject.framework";
	else if (osInformation.match(/64-bit/gi) != null)
		plugPlugFile = extensionPath + "/lib/PlugPlugExternalObject-Win/win64/PlugPlugExternalObject.dll";
	else
		plugPlugFile = extensionPath + "/lib/PlugPlugExternalObject-Win/win32/PlugPlugExternalObject.dll";
	
	plugPlugFile = escape(plugPlugFile);
	cs.evalScript(CONSTANTS.IDSN_JSX + 'loadPlugPlugLibrary()');
}

/*******************************************************************************
 * Method to add addEventListener specific to InDesign
 ******************************************************************************/
MainScript.prototype.addIDSNEventListeners = function(cs)
{
	console.log('adding listeners for indesign app');
	
	var eventListenerNames = ["DAM_CONNECT_AFTER_CLOSE"];
	cs.evalScript(CONSTANTS.IDSN_JSX + 'removeEventListeners('+  JSON.stringify(eventListenerNames) + ')');
	this.addBeforeCloseEventListener(cs);
}

/*******************************************************************************
 * Method to add Before Close EventListener
 ******************************************************************************/
MainScript.prototype.addBeforeCloseEventListener = function(cs)
{
	console.log('adding before close event listener');
	cs.addEventListener("DAM_CONNECT_DOCUMENT_BEFORE_CLOSE", function(event)
	{
		new MainScript().idsnDocumentCloseHandler(event);
	});

	cs.evalScript(CONSTANTS.IDSN_JSX + 'addBeforeCloseEventListener()');
}

/*******************************************************************************
 * Response Handler for InDesign Document Before Close Event
 ******************************************************************************/
MainScript.prototype.idsnDocumentCloseHandler = function(res)
{
	console.log("inside handler of idsn document close event");
	console.log(res);

	var objMessage = [];
	objMessage.event_type = "damconnect_before_doc_close"
	objMessage.asset_id = res.data.assetId;
	objMessage.asset_name = res.data.assetName;
	objMessage.asset_data = res.data.assetData;
	objMessage.asset_version = res.data.assetVersion;
	objMessage.asset_docPath = res.data.assetDocumentPath;
	objMessage.asset_docName = res.data.assetDocumentName;

	console.log(objMessage);
	var iframe = document.getElementById('iframe');
	iframe.contentWindow.postMessage(objMessage, "*");
}

/* -----------------INDESIGN SPECIFIC CODE ENDS HERE ------------------------------*/

/* --------------------------------------------------------------------------------*/

/* -----------------PHOTOSHOP SPECIFIC CODE STARTS HERE ---------------------------*/
/*******************************************************************************
 * Method to add addEventListener specific to InDesign
 ******************************************************************************/
MainScript.prototype.addPHXSEventListeners = function(cs, extensionId)
{
	console.log('adding listeners for photoshop app');
	var appId = cs.getApplicationID();

	var event = new CSEvent("com.adobe.PhotoshopRegisterEvent", "APPLICATION");
	event.extensionId = extensionId;  
	event.appId =  appId;

	console.log("Registering PSEvent.CLOSE");
	event.data = PSEvent.CLOSE;
	cs.dispatchEvent(event);
}

MainScript.prototype.handlePhotoshopEvent = function(event)
{
	var data = event.data;
	console.log(data);
	
	if(!(data && data.length))
		return;
	
	data = JSON.parse(data.substring(data.indexOf(',') + 1));
	
	var mainScript = new MainScript();
	var eventData = data.eventData;
	
	switch(data.eventID.toString())
	{
		case "1131180832" : mainScript.psdDocumentCloseHandler(eventData);
		break;
	}
}

/*******************************************************************************************
Method to handle PSEvent.CLOSE
********************************************************************************************/
MainScript.prototype.psdDocumentCloseHandler = function(data)
{
	console.log("inside handler of psd document close event");
	console.log(data);
}

/* -----------------PHOTOSHOP SPECIFIC CODE ENDS HERE -----------------------------*/

/*******************************************************************************
 * To create flyout menu
 ******************************************************************************/
MainScript.prototype.createMenuXML = function(cs)
{
	console.log('creating xml for settings menu');
	
	var flyoutXML = '<Menu>';
	flyoutXML += '\
        <MenuItem Id="settings" Label="Settings" Enabled="true"/> \
    	<MenuItem Id="upload" Label="Upload" Enabled="true"/> \
    </Menu>';
	
	cs.setPanelFlyoutMenu(flyoutXML);
	cs.addEventListener( CONSTANTS.flyoutMenuClicked, this.flyoutMenuClickedHandler);
}

/*******************************************************************************
 * To FlyOut menu click handler
 ******************************************************************************/
MainScript.prototype.flyoutMenuClickedHandler = function(event)
{
	console.log(event);
	switch (event.data.menuId)
	{
		case "settings":
			CONSTANTS.MAIN_SCRIPT.showSettingsDialog();
			break;
			
		case "upload":
			CONSTANTS.MAIN_SCRIPT.showUploadDialog();
			break;
	}
}

/*******************************************************************************
 * To show Dialog box to save settings
 ******************************************************************************/
MainScript.prototype.showSettingsDialog = function()
{
	console.log('settings clicked');
	var cs = CONSTANTS.CS;
	
	var extScript = CONSTANTS.COMMON_JSX + 'showSettingsDialog()';
	
	this.evalScript(extScript, function(resp)
	{
		if(isScriptResponseValid(resp))
		{
			console.log(resp);
			resp = JSON.parse(resp);
			
			if (resp.success)
			{
				var data = resp.data;
				CONSTANTS.DOWNLOAD_PATH = data.path;
				var defaultPagePath = cs.getSystemPath(SystemPath.EXTENSION) + "/Default.html";
				
				if(data.url == '')			
				{
					CONSTANTS.IFRAME_SRC = defaultPagePath;
					cs.resizeContent(530, 530);
				}
				else
				{
					CONSTANTS.IFRAME_SRC = data.url + '?ver=' + Math.random();
					console.log("now to resize 1240+ wide");
					cs.resizeContent(1240, 650);
				}
			}
			else
			{
				console.error('could not read settings');
				var defaultPagePath = cs.getSystemPath(SystemPath.EXTENSION) + "/Default.html";
				CONSTANTS.IFRAME_SRC = defaultPagePath;
				new CSInterface().resizeContent(530, 530);
			}		
			
			document.getElementById('iframe').src = CONSTANTS.IFRAME_SRC;
		}
		else
		{
			extScript = CONSTANTS.COMMON_JSX + 'readSettings()';
			
			new MainScript().evalScript(extScript, function(resp2)
			{
				if(!isScriptResponseValid(resp2))
				{
					console.log('nothing set in settings');
					var defaultPagePath = cs.getSystemPath(SystemPath.EXTENSION) + "/Default.html";
					console.log(defaultPagePath);
					CONSTANTS.IFRAME_SRC = defaultPagePath;
					document.getElementById('iframe').src = defaultPagePath;
					new CSInterface().resizeContent(530, 530);
				}
			});
		}
	});
}

/*******************************************************************************
 * To show Dialog box to save settings
 ******************************************************************************/
MainScript.prototype.showUploadDialog = function()
{
	var extScript = undefined;
	switch(CONSTANTS.CURRENT_APP)
	{
		case "IDSN" : extScript = CONSTANTS.IDSN_JSX + 'showUploadDialog()';
			break;
		
		case "PHSP" :
		case "PHXS" : extScript = CONSTANTS.PHXS_JSX + 'showUploadDialog()';
			break;
	}
	
	this.evalScript(extScript, function(resp)
	{
		console.log(resp);
		if (isScriptResponseValid(resp))
		{
			var objMessage = [];
			var data = JSON.parse(resp);
			objMessage.event_type = "damconnect_upload"
			objMessage.asset_id = data.assetId;
			objMessage.asset_name = data.assetName;
			objMessage.asset_data = data.assetData;
			objMessage.asset_version = data.assetVersion;
			objMessage.asset_docName = data.assetDocumentName;
			objMessage.asset_docPath = data.assetDocumentPath;

			document.getElementById('iframe').contentWindow.postMessage(objMessage, "*");
			
			//CONSTANTS.MAIN_SCRIPT.uploadDocument();
		}
	})
}

/*******************************************************************************
 * Method to read setting file to fetch the URL and download path
 ******************************************************************************/
MainScript.prototype.readSettings = function()
{
	console.log('inside readSettings');
	
	var extScript = CONSTANTS.COMMON_JSX + 'readSettings()';
	this.evalScript(extScript, function(resp)
	{
		console.log(resp);
		var cs = new CSInterface();
		if (isScriptResponseValid(resp))
		{
			var data = JSON.parse(resp);
			CONSTANTS.DOWNLOAD_PATH = data.path;
			
			var defaultPagePath = cs.getSystemPath(SystemPath.EXTENSION) + "/Default.html";
			if(data.url == '')			
			{
				CONSTANTS.IFRAME_SRC = defaultPagePath;
				cs.resizeContent(530, 530);
			}
			else
			{
				CONSTANTS.IFRAME_SRC = data.url + '?ver=' + Math.random();
				console.log("now to resize 1240+ wide");
				cs.resizeContent(1240, 650);
			}
		}
		else
		{
			console.log('nothing set in settings');
			CONSTANTS.MAIN_SCRIPT.showSettingsDialog();
			cs.resizeContent(530, 530);
		}
		
		document.getElementById('iframe').src = CONSTANTS.IFRAME_SRC;
	});
}
/*******************************************************************************
 * Method to evaluate the method of JSX files
 ******************************************************************************/
MainScript.prototype.evalScript = function(script, callback)
{
	new CSInterface().evalScript(script, callback);
}

/*******************************************************************************
 * Intermediate method to call JSX method to open new InDesign document
 ******************************************************************************/
MainScript.prototype.openDocument = function(path, assetInfo)
{
	var extScript = CONSTANTS.COMMON_JSX + 'openDocument(' + JSON.stringify(path) + ',' + JSON.stringify(assetInfo) + ')';
	this.evalScript(extScript, function(resp)
	{
		console.log("InDesign Document Open Successfully");
	});
}

MainScript.prototype.uploadDocument = function(eventData)
{
	console.log("After upload approved from server");

	var cs = CONSTANTS.CS;
	
	var script = '$._ext_common.getActiveDocumentPath()';
	cs.evalScript(script, function (scriptResp)
	{
		scriptResp = JSON.parse(scriptResp);
		console.log(scriptResp);
		
		if(scriptResp.success == true)
		{
			var scriptData = scriptResp.data;
			var filePath = scriptData.filePath;
			var fs = cep_node.require('fs');
			var request = require('request');
			var fileStream = fs.createReadStream(filePath);
			
			var options = 
			{
				url : (eventData && eventData.data && eventData.data.uploadUrl) ? eventData.data.uploadUrl : API_CONSTANTS.DefaultUploadUrl,
				formData : { file: fileStream },
				headers :
				{
					'Accept': 'application/json',
					'Cache-Control': 'no-cache',
					'Pragma': 'no-cache'
				}
			};
			
			request.post(options, function (err, httpResponse, body)
			{
				if (err)
					return window.console.error('upload failed:', err);
				else if(body)
				{
					var response = JSON.parse(body);
					if (response.error)
					{
						var errorObj = response.error;
						console.error(errorObj);						
					}
					else
						alert('Document Uploaded Successfully - ' + scriptData.fileName);						
				}
				else
					alert("no response from server");
				
			});
		}
		else
			alert(scriptResp.data);		
	});	
}

function isScriptResponseValid(resp)
{
	return (resp && resp.length && resp != "undefined" && resp != CONSTANTS.CS.EvalScript_ErrMessage);
}