$._ext_IDSN = {
		
	/*******************************************************************************************
        Method to load Plug Plug Library for handling of Indesign Events
    ********************************************************************************************/
    loadPlugPlugLibrary: function()
    {
        try
        {
            new ExternalObject("lib:PlugPlugExternalObject");
            var data = "PlugPlug library successfully loaded!";
            var csxsEvent = createCSXSEvent("plugPlugSuccess", data);
            csxsEvent.dispatch();
        }
        catch (error)
        {
            alert(error);
        }
    },
    
    /*******************************************************************************************
    Method to remove attached event listener with Indesign
    ********************************************************************************************/
    removeListeners : function(eventListenerNames)
    {
    	var eventListener;
        var data = '';
        
        if (app.eventListeners.length <= 0)
            return;
        
        for (var counter = 0; counter < eventListenerNames.length; counter++)
        {
        	eventListener = app.eventListeners.itemByName(eventListenerNames[counter]);
        	eventListener.isValid && eventListener.remove();
        }
    },

    /*******************************************************************************************
    Method to register document BeforeClose event
	********************************************************************************************/
	addBeforeCloseEventListener: function()
	{
		var eventListener = app.addEventListener(Document.BEFORE_CLOSE, this.handleBeforeClose);
	    eventListener.name = "DAM_CONNECT_BEFORE_CLOSE";
	    var data = "BEFORE_CLOSE event listener added.";
	    var csxsEvent = createCSXSEvent("idListenersAdded", data);
	    csxsEvent.dispatch();
	},
	
	/*******************************************************************************************
    Response Handler for document BeforeClose event
    ********************************************************************************************/
    handleBeforeClose: function(e)
    {
    	try
    	{

	    	var parent = e.parent;
	        while (!(parent instanceof Document))
	        {
	            parent = parent.parent;
	        }
	        
	        var docRef = parent;
	        var data = {};
	        data.assetId = docRef.extractLabel('damconnect_assetId');
	        data.assetName = docRef.extractLabel('damconnect_assetName');  
	        data.assetData = docRef.extractLabel('damconnect_metadata');   
	        data.assetVersion = docRef.extractLabel('damconnect_versionId');
	    	data.assetDocumentName = docRef.name;
			data.assetDocumentPath = '';
			
	    	try
	    	{
	    		data.assetDocumentPath = docRef.fullName.absoluteURI;
	    	}
	    	catch(ex) // a new document case, without any name and file path
	    	{
	    	}
	    	 
	        var csxsEvent = createCSXSEvent("DAM_CONNECT_DOCUMENT_BEFORE_CLOSE", JSON.stringify(data));
	        csxsEvent.dispatch();
        }
        catch(e) { 
        	//alert(e.message);
        }
	},
	
	/*******************************************************************************************
    Method to show dialog to set iframe src and download path
    ********************************************************************************************/
    showUploadDialog: function()
    {
    	if(app.documents.length > 0)
    	{    
	        var w = new Window("dialog", "Upload");
	               
	        var grp1 = w.add("group")
	        var _url_label = grp1.add("statictext");              
	        _url_label.text = "Do you want to upload the current active document?";
	        
	        var grp2 = w.add("group")
	        grp2.alignment = "right";
	        var _ok_btn = grp2.add("button", undefined, "OK");
	        var _cancel_btn = grp2.add("button", undefined, "Cancel");
	        
	        with(w) { _cancel_btn.onClick = function() { close(null); } }
	        
	        if (w.show() == 1) {
	        
        		var docRef = app.activeDocument;
        		var data = new Object();
        		data.assetId = docRef.extractLabel('damconnect_assetId');
        		data.assetName = docRef.extractLabel('damconnect_assetName');  
        		data.assetData = docRef.extractLabel('damconnect_metadata');   
        		data.assetVersion = docRef.extractLabel('damconnect_versionId');  
        		data.assetDocumentPath = docRef.fullName.absoluteURI; 
        		data.assetDocumentName = docRef.name;	        
	        
				return JSON.stringify(data);
	        }
        }
    },
    
    /*******************************************************************************************
    Method to open InDesign document
    ********************************************************************************************/
    openDocument: function(assetInfo)
    {
        var fileRef = new File(assetInfo.path);
        var docRef = app.open(fileRef);
        
        docRef.insertLabel('damconnect_assetName', assetInfo.assetName);
        docRef.insertLabel('damconnect_versionId', assetInfo.version);
        docRef.insertLabel('damconnect_assetId', assetInfo.assetId);
        
        docRef.save();
    },
    
    /*******************************************************************************************
    Method to place any image asset file onto InDesign document
    ********************************************************************************************/
    placeFile: function(path)
    {
    	var retVal = { success : false };
    	
    	try
    	{
    		if(app.documents.length == 0)
        	app.documents.add();

	    	var myFile = new File(path);
	    	app.activeDocument.placeGuns.loadPlaceGun(myFile);
	    	retVal.success = true;
    	}
    	catch(ex)
    	{
    		retVal.data = ex.message;
    	}
    	finally
    	{
    		return JSON.stringify(retVal);
    	}
    	
	}
};

/*******************************************************************************************
Method to return CSXSEvent object
********************************************************************************************/
function createCSXSEvent(type, data)
{
	var csxsEvent = new CSXSEvent();
	csxsEvent.type = type;
	csxsEvent.data = data;
	return csxsEvent;
}