﻿if (typeof xmpModifier !== 'object') {
    xmpModifier = {};
}

/**
 * This method will return xmp of the active document and register the namespace if not exist
 * @param {[type]} nameSpace [namespace for SLS ]
 * @param {[type]} nodeName  [shortform for namespace]
 */
function GetXMP(nameSpace, nodeName, document, activeLayer) {
    var context = this;
    context.destNamespace = nameSpace;
    context.nodeName = nodeName;
    context.document = document || app.activeDocument;
    context.activeLayer = activeLayer || app.activeDocument.activeLayer;

    context.init = function () {
        var activeLayerXMP = null;
        var documentXMP = null
        if (ExternalObject.AdobeXMPScript == undefined) {
            ExternalObject.AdobeXMPScript = new ExternalObject("lib:AdobeXMPScript");
        }

        var registeredNameSpace = XMPMeta.registerNamespace(context.destNamespace, context.nodeName);
        var prefix = XMPMeta.getNamespacePrefix(context.destNamespace);
        var isValidXmp = false;
        //~         if (prefix != '' && activeLayer == null) {
        //~             documentXMP = new XMPMeta(context.document.xmpMetadata.rawData);
        //~             isValidXmp = true;
        //~         } else if (prefix != '' && context.activeLayer) {
        //~             try {
        //~                 activeLayerXMP = new XMPMeta(context.activeLayer.xmpMetadata.rawData);
        //~             } catch (e) {
        //~                 activeLayerXMP = new XMPMeta()
        //~             }
        //~             isValidXmp = true;
        //~         }
        if (prefix != '') {
            documentXMP = new XMPMeta(context.document.xmpMetadata.rawData);
            try {
                activeLayerXMP = new XMPMeta(context.activeLayer.xmpMetadata.rawData);
            } catch (e) {
                activeLayerXMP = new XMPMeta()
            }
            isValidXmp = true;
        }
        context.documentXMP = documentXMP;
        context.activeLayerXMP = activeLayerXMP;
        context.isXmpCreated = isValidXmp;
    }

	/**
	 * Method to check whether struct field exist in document xmp or not
	 * @param  {[type]} structFieldName [name of the struct field]
	 * @return {[type]}                 [return true if exist and false if not exist]
	 */
    context.doesStructFieldExist = function (structFieldName) {
        return context.documentXMP.doesStructFieldExist(context.destNamespace, context.nodeName, context.destNamespace, structFieldName)
    }

	/**
	 * Add or Update values for structfield
	 * @param {[type]} structFieldName  [name of the struct field]
	 * @param {[type]} structFieldValue [value for the struct field]
	 * @param {[type]} doesStringify    [flag whether value need to be stored as string or not]
	 */
    context.setStructField = function (structFieldName, structFieldValue, doesStringify) {
        structFieldValue = doesStringify ? JSON.stringify(structFieldValue) : structFieldValue;
        context.documentXMP.setStructField(context.destNamespace, context.nodeName, context.destNamespace, structFieldName, structFieldValue);
        context.commitDocumentXMP();
    }

	/**	
	 * Method to get values for strcut field stored in document xmp
	 * @param  {[type]} structFieldName [name of the struct field whose value need to be fetch]
	 * @param  {[type]} doesParse       [flag, if true then return value of the struct will be parse using JSON.parse()]
	 * @return {[type]}                 [value for the struct filed stored in the document xmp.]
	 */
    context.getStructField = function (structFieldName, doesParse) {
        var structFieldValue = [];
        if (context.doesStructFieldExist(structFieldName)) {
            structFieldValue = context.documentXMP.getStructField(context.destNamespace, context.nodeName, context.destNamespace, structFieldName).value;
            structFieldValue = doesParse ? JSON.parse(structFieldValue) : structFieldValue;
        }
        return structFieldValue;
    }

	/**	
	 * Method to delete the complete struct field
	 * @param  {[type]} structFieldName [name of the the struct field that need to be deleted]
	 * @return {[type]}                 [description]
	 */
    context.deleteStructField = function (structFieldName) {
        if (context.doesStructFieldExist(structFieldName)) {
            context.documentXMP.deleteStructField(context.destNamespace, context.nodeName, context.destNamespace, structFieldName);
            context.commitDocumentXMP();
        }
    }

    /**
     * Method to check whether struct field exist in active layer xmp or not
     * @param {*} structFieldName 
     */
    context.doesStructFieldExistInActiveLayer = function (structFieldName) {
        return context.activeLayerXMP.doesStructFieldExist(context.destNamespace, context.nodeName, context.destNamespace, structFieldName)
    }

    /**
     * Add or Update values for structfield in active layer
     * @param {*} structFieldName 
     * @param {*} structFieldValue 
     * @param {*} doesStringify 
     */
    context.setActiveLayerStructField = function (structFieldName, structFieldValue, doesStringify) {
        structFieldValue = doesStringify ? JSON.stringify(structFieldValue) : structFieldValue;
        context.activeLayerXMP.setStructField(context.destNamespace, context.nodeName, context.destNamespace, structFieldName, structFieldValue);
        context.commitActiveLayerXMP();
    }

    /**
     * Method to get structfiled from active layer
     * @param {*} structFieldName 
     * @param {*} doesParse 
     */
    context.getActiveLayerStructField = function (structFieldName, doesParse) {
        var structFieldValue = [];
        if (context.doesStructFieldExistInActiveLayer(structFieldName)) {
            structFieldValue = context.activeLayerXMP.getStructField(context.destNamespace, context.nodeName, context.destNamespace, structFieldName).value;
            structFieldValue = doesParse ? JSON.parse(structFieldValue) : structFieldValue;
        }
        return structFieldValue;
    }

	/**
	 * Method to add all changes done in xmp back. This method will serialize the document xmp
	 * @return {[type]} [return new document xmp and stored back.]
	 */
    context.commitDocumentXMP = function () {
        var packet = context.documentXMP.serialize(XMPConst.SERIALIZE_USE_COMPACT_FORMAT);
        context.document.xmpMetadata.rawData = packet;
        context.document.saved = false;
    }

    /**
	 * Method to add all changes done in xmp back. This method will serialize the activelayer xmp
	 * @return {[type]} [return new document xmp and stored back.]
	 */
    context.commitActiveLayerXMP = function () {
        var packet = context.activeLayerXMP.serialize(XMPConst.SERIALIZE_USE_COMPACT_FORMAT);
var a = context;     
       context.activeLayer.xmpMetadata.rawData = packet;
        context.document.saved = false;
    }

    context.init();
}

xmpModifier.GetXMP = GetXMP;